/**************************************************************************
 *  Copyright (C) 2008 by Simon Qian                                      *
 *  SimonQian@SimonQian.com                                               *
 *                                                                        *
 *  Project:    vsprog                                                    *
 *  File:       programmer.h                                              *
 *  Author:     SimonQian                                                 *
 *  Versaion:   See changelog                                             *
 *  Purpose:    common functions for all programmers                      *
 *  License:    See license                                               *
 *------------------------------------------------------------------------*
 *  Change Log:                                                           *
 *      YYYY-MM-DD:     What(by Who)                                      *
 *      2008-10-29:     created(by SimonQian)                             *
 **************************************************************************/
#define PROGRAMMER_DEFAULT				0

typedef struct
{
	// the 3 element listed below MUST be define valid
	// other functions can be initialized in .init_ability()
	char *name;
	RESULT (*parse_argument)(char cmd, const char **argu, uint32 argu_num);
	RESULT (*init_capability)(void *p);

	// init and fini
	RESULT (*init)(void);
	RESULT (*fini)(void);

	// interfaces supported
	uint32 interfaces;

	// peripheral
	// spi
	RESULT (*spi_init)(void);
	RESULT (*spi_fini)(void);
	RESULT (*spi_config)(uint16 kHz, uint8 cpol, uint8 cpha, uint8 first_bit);
	RESULT (*spi_io)(uint8 *out, uint8 *in, uint16 len, uint16 inpos, uint16 inlen);

	// gpio
	RESULT (*gpio_init)(void);
	RESULT (*gpio_fini)(void);
	RESULT (*gpio_config)(uint16 pin_mask, uint16 io);
	RESULT (*gpio_out)(uint16 pin_mask, uint16 value);
	RESULT (*gpio_in)(uint16 pin_mask, uint16 *value);

	// delay
	RESULT (*delayms)(uint16 ms);
	RESULT (*delayus)(uint16 us);

	RESULT (*peripheral_commit)(void);

	// target
	RESULT (*get_target_voltage)(uint16 *voltage);

	// issp for PSoC
	RESULT (*issp_init)(void);
	RESULT (*issp_fini)(void);
	RESULT (*issp_enter_program_mode)(uint8 mode);
	RESULT (*issp_leave_program_mode)(uint8 mode);
	RESULT (*issp_wait_and_poll)(void);
	RESULT (*issp_vector)(uint8 operate, uint8 addr, uint8 data, uint8 *buf);
	RESULT (*issp_commit)(void);

	// jtag
	RESULT (*jtag_init)(void);
	RESULT (*jtag_fini)(void);
	RESULT (*jtag_config)(uint16 kHz, uint8 ub, uint8 ua, uint16 bb, uint16 ba);
	RESULT (*jtag_hl_tms)(uint8 *tms, uint8 len);
	RESULT (*jtag_hl_ir)(uint8 *ir, uint8 len, uint8 idle, uint8 want_ret);
	RESULT (*jtag_hl_dr)(uint8 *dr, uint16 len, uint8 idle, uint8 want_ret);
/*
	RESULT (*jtag_hl_poll)(uint8 *ir0, uint8 ir0idle, uint8 ir0len, 
						   uint8 *dr0, uint8 dr0idle, uint8 dr0len,
						   uint8 *ir1, uint8 ir1idle, uint8 ir1len, 
						   uint8 *dr1, uint8 dr1idle, uint8 dr1len,
						   uint8 *ir2, uint8 ir2idle, uint8 ir2len, 
						   uint8 *dr2, uint8 dr2idle, uint8 dr2len,
						   uint8 pos, uint8 mask, uint8 value, uint16 poll_count);
*/
	RESULT (*jtag_hl_delay_us)(uint16 us);
	RESULT (*jtag_hl_delay_ms)(uint16 ms);
	RESULT (*jtag_aux_io_init)(void);
	RESULT (*jtag_aux_io_fini)(void);
	RESULT (*jtag_aux_io_config)(uint8 pin_mask, uint8 io);
	RESULT (*jtag_aux_io_out)(uint8 pin_mask, uint8 value);
	RESULT (*jtag_aux_io_in)(uint8 pin_mask, uint8 *value);
	RESULT (*jtag_hl_commit)(void);

	// sbw <==> Spy-Bi-Wire
	RESULT (*sbw_init)(void);
	RESULT (*sbw_fini)(void);
	RESULT (*sbw_config)(uint16 kHz, uint8 ub, uint8 ua, uint16 bb, uint16 ba);
	RESULT (*sbw_hl_tms)(uint8 *tms, uint8 len);
	RESULT (*sbw_hl_ir)(uint8 *ir, uint8 len, uint8 idle);
	RESULT (*sbw_hl_dr)(uint8 *dr, uint16 len, uint8 idle);
	RESULT (*sbw_hl_delay_us)(uint16 us);
	RESULT (*sbw_hl_delay_ms)(uint16 ms);
	RESULT (*sbw_aux_io_config)(uint16 pin_mask, uint16 io);
	RESULT (*sbw_aux_io_out)(uint16 pin_mask, uint16 value);
	RESULT (*sbw_aux_io_in)(uint16 pin_mask, uint16 *value);
	RESULT (*sbw_hl_commit)(void);

	// c2 for c8051f
	RESULT (*c2_init)(void);
	RESULT (*c2_fini)(void);
	RESULT (*c2_addr_read)();
	RESULT (*c2_addr_write)();
	RESULT (*c2_data_read)();
	RESULT (*c2_data_write)();
	RESULT (*c2_commit)();

	// mass-product support
	RESULT (*query_mass_product_data_size)(uint32 *size);
	RESULT (*download_mass_product_data)(uint8 *buffer, uint32 len);
}programmer_info_t;


extern programmer_info_t *cur_programmer;
extern programmer_info_t programmers_info[];


void programmer_print_list(void);
RESULT programmer_init(const char *programmer, const char *app_dir);

