/**************************************************************************
 *  Copyright (C) 2008 by Simon Qian                                      *
 *  SimonQian@SimonQian.com                                               *
 *                                                                        *
 *  Project:    vsprog                                                    *
 *  File:       usbtospi.c                                                *
 *  Author:     SimonQian                                                 *
 *  Versaion:   See changelog                                             *
 *  Purpose:    usb_to_spi driver for Versaloon                           *
 *  License:    See license                                               *
 *------------------------------------------------------------------------*
 *  Change Log:                                                           *
 *      YYYY-MM-DD:     What(by Who)                                      *
 *      2008-10-29:     created(by SimonQian)                             *
 **************************************************************************/
#include "app_cfg.h"
#include "usbtoxxx.h"

uint8 usbtogpio_num_of_interface;

RESULT usbtogpio_init(void)
{
	return usbtoxxx_init_command(USB_TO_GPIO, &usbtogpio_num_of_interface);
}

RESULT usbtogpio_fini(void)
{
	return usbtoxxx_fini_command(USB_TO_GPIO);
}

RESULT usbtogpio_config(uint8 interface_index, uint16 mask, uint16 direction)
{
	uint8 conf[3];

	conf[0] = (mask >> 0) & 0xFF;
	conf[1] = (mask >> 8) & 0xFF;
	if(direction > 0)
	{
		conf[2] = USB_TO_GPIO_DIR_MSK;
	}
	else
	{
		conf[2] = 0;
	}
	return usbtoxxx_conf_command(USB_TO_GPIO, interface_index, conf, 3);
}

RESULT usbtogpio_in(uint8 interface_index, uint16 mask, uint16 *value)
{
	uint8 buf[2];

	buf[0] = (mask >> 0) & 0xFF;
	buf[1] = (mask >> 8) & 0xFF;
	return usbtoxxx_in_command(USB_TO_GPIO, interface_index, buf, 2, 2, (uint8*)value, 0, 2, 0);
}

RESULT usbtogpio_out(uint8 interface_index, uint16 mask, uint16 value)
{
	uint8 buf[4];

	buf[0] = (mask >> 0) & 0xFF;
	buf[1] = (mask >> 8) & 0xFF;
	buf[2] = (value >> 0) & 0xFF;
	buf[3] = (value >> 8) & 0xFF;
	return usbtoxxx_out_command(USB_TO_GPIO, interface_index, buf, 4, 0);
}

