/**************************************************************************
 *  Copyright (C) 2008 by Simon Qian                                      *
 *  SimonQian@SimonQian.com                                               *
 *                                                                        *
 *  Project:    vsprog                                                    *
 *  File:       usbtoissp.c                                               *
 *  Author:     SimonQian                                                 *
 *  Versaion:   See changelog                                             *
 *  Purpose:    usb_to_issp driver for Versaloon                          *
 *  License:    See license                                               *
 *------------------------------------------------------------------------*
 *  Change Log:                                                           *
 *      YYYY-MM-DD:     What(by Who)                                      *
 *      2008-10-29:     created(by SimonQian)                             *
 **************************************************************************/
#include "app_cfg.h"
#include "usbtoxxx.h"

uint8 usbtoissp_num_of_interface = 0;


RESULT usbtoissp_init(void)
{
	return usbtoxxx_init_command(USB_TO_ISSP, &usbtoissp_num_of_interface);
}

RESULT usbtoissp_fini(void)
{
	return usbtoxxx_fini_command(USB_TO_ISSP);
}

RESULT usbtoissp_config(uint8 interface_index)
{
#ifdef PARAM_CHECK
	if(interface_index > 7) { LOG_BUG("invalid inteface_index %d\n", interface_index); return ERROR_FAIL; }
#endif

	return usbtoxxx_conf_command(USB_TO_ISSP, interface_index, NULL, 0);
}

RESULT usbtoissp_vector(uint8 interface_index, uint8 operate, uint8 addr, uint8 data, uint8 *buf)
{
	uint8 cmd_buf[3];

#ifdef PARAM_CHECK
	if(interface_index > 7) { LOG_BUG("invalid inteface_index %d\n", interface_index); return ERROR_FAIL; }
#endif

	cmd_buf[0] = operate;
	cmd_buf[1] = addr;
	cmd_buf[2] = data;

	if(operate & ISSP_VECTOR_ATTR_READ)
	{
		return usbtoxxx_inout_command(USB_TO_ISSP, interface_index, cmd_buf, 3, 1, buf, 0, 1, 1);
	}
	else
	{
		return usbtoxxx_inout_command(USB_TO_ISSP, interface_index, cmd_buf, 3, 0, NULL, 0, 0, 1);
	}
}

RESULT usbtoissp_enter_program_mode(uint8 interface_index, uint8 mode)
{
#ifdef PARAM_CHECK
	if(interface_index > 7) { LOG_BUG("invalid inteface_index %d\n", interface_index); return ERROR_FAIL; }
#endif

	return usbtoxxx_in_command(USB_TO_ISSP, interface_index, &mode, 1, 0, NULL, 0, 0, 0);
}

RESULT usbtoissp_leave_program_mode(uint8 interface_index, uint8 mode)
{
#ifdef PARAM_CHECK
	if(interface_index > 7) { LOG_BUG("invalid inteface_index %d\n", interface_index); return ERROR_FAIL; }
#endif

	return usbtoxxx_out_command(USB_TO_ISSP, interface_index, &mode, 1, 0);
}

RESULT usbtoissp_wait_and_poll(uint8 interface_index)
{
#ifdef PARAM_CHECK
	if(interface_index > 7) { LOG_BUG("invalid inteface_index %d\n", interface_index); return ERROR_FAIL; }
#endif

	return usbtoxxx_poll_command(USB_TO_ISSP, interface_index, NULL, 0);
}

