/**************************************************************************
 *  Copyright (C) 2008 by Simon Qian                                      *
 *  SimonQian@SimonQian.com                                               *
 *                                                                        *
 *  Project:    vsprog                                                    *
 *  File:       usbtospi.c                                                *
 *  Author:     SimonQian                                                 *
 *  Versaion:   See changelog                                             *
 *  Purpose:    usb_to_spi driver for Versaloon                           *
 *  License:    See license                                               *
 *------------------------------------------------------------------------*
 *  Change Log:                                                           *
 *      YYYY-MM-DD:     What(by Who)                                      *
 *      2008-10-29:     created(by SimonQian)                             *
 **************************************************************************/
#include "app_cfg.h"
#include "usbtoxxx.h"

uint8 usbtospi_num_of_interface = 0;


RESULT usbtospi_init(void)
{
	return usbtoxxx_init_command(USB_TO_SPI, &usbtospi_num_of_interface);
}

RESULT usbtospi_fini(void)
{
	return usbtoxxx_fini_command(USB_TO_SPI);
}

RESULT usbtospi_config(uint8 interface_index, uint16 freq, uint8 cpol, uint8 cpha, uint8 firstbit)
{
	uint8 conf[3];

#ifdef PARAM_CHECK
	if(interface_index > 7) { LOG_BUG("invalid inteface_index %d\n", interface_index); return ERROR_FAIL; }
#endif

	conf[0] = cpol | cpha | firstbit;
	conf[1] = (freq >> 0) & 0xFF;
	conf[2] = (freq >> 8) & 0xFF;
	return usbtoxxx_conf_command(USB_TO_SPI, interface_index, conf, 3);
}

RESULT usbtospi_io(uint8 interface_index, uint8 *out, uint8 *in, uint16 outlen, uint16 inpos, uint16 inlen)
{
#ifdef PARAM_CHECK
	if(interface_index > 7) { LOG_BUG("invalid inteface_index %d\n", interface_index); return ERROR_FAIL; }
#endif

	return usbtoxxx_inout_command(USB_TO_SPI, interface_index, out, outlen, outlen, in, inpos, inlen, 1);
}

