/**************************************************************************
 *  Copyright (C) 2008 by Simon Qian                                      *
 *  SimonQian@SimonQian.com                                               *
 *                                                                        *
 *  Project:    vsprog                                                    *
 *  File:       usbtoxxx.c                                                *
 *  Author:     SimonQian                                                 *
 *  Versaion:   See changelog                                             *
 *  Purpose:    usb_to_xxx driver for Versaloon                           *
 *  License:    See license                                               *
 *------------------------------------------------------------------------*
 *  Change Log:                                                           *
 *      YYYY-MM-DD:     What(by Who)                                      *
 *      2008-10-29:     created(by SimonQian)                             *
 **************************************************************************/
#include "app_cfg.h"
#include "usbtoxxx.h"

const char* types_name[96] =	{"usbtousart", "usbtospi", "usbtoi2c", "usbtogpio", "usbtocan", "usbtopwm","usbtoadc", "usbtodac", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "usbtopower", "usbtodelay", 
								 "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", 
								 "usbtojtagll", "usbtojtaghl", "usbtoissp", "usbtoc2", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", 
								 "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", 
								 "usbtopower", "usbtodelay", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "usbtoall"};
#define usbtoxxx_get_type_name(type)				types_name[((type) - VERSALOON_USB_TO_XXX_CMD_START) % (sizeof(types_name) / sizeof(types_name[0]))]

static uint8 type_pre = 0;
static uint16 usbtoxxx_buffer_index = 0;
static uint16 usbtoxxx_current_cmd_index = 0;
static uint8 *usbtoxxx_buffer = NULL;
uint8 null_char = 0;

uint16 collect_index = 0;
uint8 collect_cmd;

RESULT usbtoxxx_validate_current_command_type(void)
{
	if(type_pre > 0)
	{
		// not the first command
		if(NULL == usbtoxxx_buffer) { LOG_ERROR("there should be bugs in the code, report to author\n"); return ERROR_FAIL; }

		usbtoxxx_buffer[0] = type_pre;
		usbtoxxx_buffer[1] = (usbtoxxx_current_cmd_index >> 0) & 0xFF;
		usbtoxxx_buffer[2] = (usbtoxxx_current_cmd_index >> 8) & 0xFF;

		usbtoxxx_buffer_index += usbtoxxx_current_cmd_index;
	}
	else
	{
		// first command
		usbtoxxx_buffer_index = 3;
	}

	// prepare for next command
	usbtoxxx_current_cmd_index = 3;
	usbtoxxx_buffer = versaloon_buf + usbtoxxx_buffer_index;

	collect_index = 0;
	collect_cmd = 0;

	return ERROR_OK;
}



RESULT usbtoxxx_execute_command(void)
{
	uint16 i;
	uint16 inlen;

	if(ERROR_OK != usbtoxxx_validate_current_command_type()) { LOG_BUG("can usbtoxxx_validate_current_command_type fail?\n"); return ERROR_FAIL; }

	versaloon_buf[0] = USB_TO_ALL;
	versaloon_buf[1] = (usbtoxxx_buffer_index >> 0) & 0xFF;
	versaloon_buf[2] = (usbtoxxx_buffer_index >> 8) & 0xFF;
	if(ERROR_OK != versaloon_send_command(usbtoxxx_buffer_index, &inlen)) { return ERROR_FAIL; }

	// process return data
	usbtoxxx_buffer_index = 0;
	for(i = 0; i < versaloon_pending_idx; i++)
	{
		// check result
		if((0 == i) || !((versaloon_pending_result[i].collect) && (versaloon_pending_result[i - 1].collect) && (versaloon_pending_result[i].cmd == versaloon_pending_result[i - 1].cmd)))
		{
			if(USB_TO_XXX_SUCCESS != versaloon_buf[usbtoxxx_buffer_index])
			{
				LOG_ERROR("%s command 0x%02x failed with 0x%02x\n", 
					usbtoxxx_get_type_name(versaloon_pending_result[i].type), versaloon_pending_result[i].cmd, versaloon_buf[usbtoxxx_buffer_index]);
				return ERROR_FAIL;
			}
			usbtoxxx_buffer_index++;
		}

		// get result data
		if((versaloon_pending_result[i].want_data_size > 0) && (versaloon_pending_result[i].data_buffer != NULL))
		{
			memcpy(versaloon_pending_result[i].data_buffer, versaloon_buf + usbtoxxx_buffer_index + versaloon_pending_result[i].want_data_pos, versaloon_pending_result[i].want_data_size);
		}
		usbtoxxx_buffer_index += versaloon_pending_result[i].actual_data_size;
		if(usbtoxxx_buffer_index > inlen) { LOG_BUG("%s command 0x%02x process error\n", usbtoxxx_get_type_name(versaloon_pending_result[i].type), versaloon_pending_result[i].cmd); return ERROR_FAIL; }
	}

	if(inlen != usbtoxxx_buffer_index) { LOG_ERROR("length of return data invalid\n"); return ERROR_FAIL; }

	if(versaloon_pending_idx > 0)
	{
		versaloon_pending_idx = 0;
	}
	else
	{
		// no receive data, avoid collision
		sleep_ms(10);
	}

	type_pre = 0;
	collect_cmd = 0;
	collect_index = 0;

	return ERROR_OK;
}

RESULT usbtoxxx_init(void)
{
	versaloon_pending_idx = 0;

	return ERROR_OK;
}

void usbtoxxx_fini(void)
{
	usbtoxxx_buffer = NULL;
	type_pre = 0;
}



RESULT usbtoxxx_add_command(uint8 type, uint8 cmd, uint8 *cmdbuf, uint16 cmdlen, uint16 retlen, uint8 *wantbuf, uint16 wantpos, uint16 wantlen, uint8 collect)
{
	uint16 len_tmp;

	if((type_pre != type) || (NULL == usbtoxxx_buffer))
	{
		if(ERROR_OK != usbtoxxx_validate_current_command_type()) { LOG_BUG("can usbtoxxx_validate_current_command_type fail?\n"); return ERROR_FAIL; }

		type_pre = type;
	}

	if((0 == collect_index) || (collect_cmd != cmd))
	{
		usbtoxxx_buffer[usbtoxxx_current_cmd_index++] = cmd;

		if(collect)
		{
			collect_index = usbtoxxx_current_cmd_index;
			collect_cmd = cmd;
		}
		else
		{
			collect_index = 0;
			collect_cmd = 0;
		}
		usbtoxxx_buffer[usbtoxxx_current_cmd_index++] = (cmdlen >> 0) & 0xFF;
		usbtoxxx_buffer[usbtoxxx_current_cmd_index++] = (cmdlen >> 8) & 0xFF;
	}
	else
	{
		len_tmp = usbtoxxx_buffer[collect_index + 0] + (usbtoxxx_buffer[collect_index + 1] << 8) + cmdlen;
		usbtoxxx_buffer[collect_index + 0] = (len_tmp >> 0) & 0xFF;
		usbtoxxx_buffer[collect_index + 1] = (len_tmp >> 8) & 0xFF;
	}

	memcpy(usbtoxxx_buffer + usbtoxxx_current_cmd_index, cmdbuf, cmdlen);
	usbtoxxx_current_cmd_index += cmdlen;

	return versaloon_add_pending(type, cmd, retlen, wantpos, wantlen, wantbuf, collect);
}





RESULT usbtodelay_delay(uint16 dly)
{
	if(ERROR_OK != usbtoxxx_validate_current_command_type()) { LOG_BUG("can usbtoxxx_validate_current_command_type fail?\n"); return ERROR_FAIL; }
	type_pre = USB_TO_DELAY;

	usbtoxxx_buffer[usbtoxxx_current_cmd_index++] = (dly >> 0) & 0xFF;
	usbtoxxx_buffer[usbtoxxx_current_cmd_index++] = (dly >> 8) & 0xFF;

	return versaloon_add_pending(USB_TO_DELAY, 0, 0, 0, 0, NULL, 0);
}

