/**************************************************************************
 *  Copyright (C) 2008 by Simon Qian                                      *
 *  SimonQian@SimonQian.com                                               *
 *                                                                        *
 *  Project:    vsprog                                                    *
 *  File:       usbtoxxx.h                                                *
 *  Author:     SimonQian                                                 *
 *  Versaion:   See changelog                                             *
 *  Purpose:    usb_to_xxx driver for Versaloon                           *
 *  License:    See license                                               *
 *------------------------------------------------------------------------*
 *  Change Log:                                                           *
 *      YYYY-MM-DD:     What(by Who)                                      *
 *      2008-10-29:     created(by SimonQian)                             *
 **************************************************************************/
#include "../versaloon.h"

// USB_TO_XXX USB Commands
// Page0
#define USB_TO_USART				(VERSALOON_USB_TO_XXX_CMD_START + 0x00)
#define USB_TO_SPI					(VERSALOON_USB_TO_XXX_CMD_START + 0x01)
#define USB_TO_I2C					(VERSALOON_USB_TO_XXX_CMD_START + 0x02)
#define USB_TO_GPIO					(VERSALOON_USB_TO_XXX_CMD_START + 0x03)
#define USB_TO_CAN					(VERSALOON_USB_TO_XXX_CMD_START + 0x04)
#define USB_TO_PWM					(VERSALOON_USB_TO_XXX_CMD_START + 0x05)
#define USB_TO_ADC					(VERSALOON_USB_TO_XXX_CMD_START + 0x06)
#define USB_TO_DAC					(VERSALOON_USB_TO_XXX_CMD_START + 0x07)
// Page1
#define USB_TO_JTAG_LL				(VERSALOON_USB_TO_XXX_CMD_START + 0x20)
#define USB_TO_JTAG_HL				(VERSALOON_USB_TO_XXX_CMD_START + 0x21)
#define USB_TO_ISSP					(VERSALOON_USB_TO_XXX_CMD_START + 0x22)
#define USB_TO_C2					(VERSALOON_USB_TO_XXX_CMD_START + 0x23)
// Page2
#define USB_TO_POWER				(VERSALOON_USB_TO_XXX_CMD_START + 0x40)
#define USB_TO_DELAY				(VERSALOON_USB_TO_XXX_CMD_START + 0x41)
#define USB_TO_ALL					(VERSALOON_USB_TO_XXX_CMD_START + 0x5F)



// USB_TO_XXX Masks
#define USB_TO_XXX_CMDMASK			0xF8
#define USB_TO_XXX_CMDSHIFT			3
#define USB_TO_XXX_IDXMASK			0x07
// USB_TO_XXX Sub Commands
// Common Sub Commands
#define USB_TO_XXX_INIT				(0x00 << USB_TO_XXX_CMDSHIFT)
#define USB_TO_XXX_FINI				(0x01 << USB_TO_XXX_CMDSHIFT)
#define USB_TO_XXX_CONFIG			(0x02 << USB_TO_XXX_CMDSHIFT)
#define USB_TO_XXX_GETHWINFO		(0x03 << USB_TO_XXX_CMDSHIFT)
#define USB_TO_XXX_STATUS			(0X04 << USB_TO_XXX_CMDSHIFT)
#define USB_TO_XXX_IN_OUT			(0x05 << USB_TO_XXX_CMDSHIFT)
#define USB_TO_XXX_IN				(0x06 << USB_TO_XXX_CMDSHIFT)
#define USB_TO_XXX_OUT				(0x07 << USB_TO_XXX_CMDSHIFT)
#define USB_TO_XXX_POLL				(0x08 << USB_TO_XXX_CMDSHIFT)
// USB_TO_JTAG_LL
#define USB_TO_JTAG_LL_SCAN			USB_TO_XXX_IN_OUT
#define USB_TO_JTAG_LL_TMS			USB_TO_XXX_OUT
// USB_TO_JTAG_HL
#define USB_TO_JTAG_HL_IR_DR		USB_TO_XXX_IN_OUT
#define USB_TO_JTAG_HL_TMS			USB_TO_XXX_OUT
// USB_TO_ISSP
#define USB_TO_ISSP_EnterProgMode	USB_TO_XXX_IN
#define USB_TO_ISSP_LeaveProgMode	USB_TO_XXX_OUT
#define USB_TO_ISSP_Vector			USB_TO_XXX_IN_OUT
#define USB_TO_ISSP_WaitAndPoll		USB_T0_XXX_POLL



// USB_TO_XXX Replys
#define USB_TO_XXX_SUCCESS			0x00
#define USB_TO_XXX_FAIL				0x08
#define USB_TO_XXX_NOT_IMPLEMENTED	0x10
#define USB_TO_XXX_PARA_ERROR		0x20
#define USB_TO_XXX_IDX_NOT_EXIST	0x28
#define USB_TO_XXX_DATA				0x80



// USB_TO_XXX
RESULT usbtoxxx_init(void);
void usbtoxxx_fini(void);
RESULT usbtoxxx_add_pending(uint8 type, uint8 cmd, uint16 actual_szie, uint16 want_pos, uint16 want_size, uint8 *buffer);
RESULT usbtoxxx_execute_command(void);

extern uint8 null_char;
RESULT usbtoxxx_add_command(uint8 type, uint8 cmd, uint8 *cmdbuf, uint16 cmdlen, uint16 retlen, uint8 *wantbuf, uint16 wantpos, uint16 wantlen, uint8 collect);
#define usbtoxxx_init_command(type, interface_num)													usbtoxxx_add_command((type), (USB_TO_XXX_INIT), &null_char, 1, 1, interface_num, 0, 1, 0)
#define usbtoxxx_fini_command(type)																	usbtoxxx_add_command((type), (USB_TO_XXX_FINI), &null_char, 1, 0, NULL, 0, 0, 0)
#define usbtoxxx_conf_command(type, port, cmdbuf, cmdlen)											usbtoxxx_add_command((type), (USB_TO_XXX_CONFIG | (port)), (cmdbuf), (cmdlen), 0, NULL, 0, 0, 0)
#define usbtoxxx_inout_command(type, port, cmdbuf, cmdlen, retlen, wantbuf, wantpos, wantlen, c)	usbtoxxx_add_command((type), (USB_TO_XXX_IN_OUT | (port)), (cmdbuf), (cmdlen), retlen, wantbuf, wantpos, wantlen, (c))
#define usbtoxxx_in_command(type, port, cmdbuf, cmdlen, retlen, wantbuf, wantpos, wantlen, c)		usbtoxxx_add_command((type), (USB_TO_XXX_IN | (port)), (cmdbuf), (cmdlen), retlen, wantbuf, wantpos, wantlen, (c))
#define usbtoxxx_out_command(type, port, cmdbuf, cmdlen, c)											usbtoxxx_add_command((type), (USB_TO_XXX_OUT | (port)), (cmdbuf), (cmdlen), 0, NULL, 0, 0, (c))
#define usbtoxxx_poll_command(type, port, cmdbuf, cmdlen)											usbtoxxx_add_command((type), (USB_TO_XXX_POLL | (port)), (cmdbuf), (cmdlen), 0, NULL, 0, 0, 0)



// USB_TO_DELAY
RESULT usbtodelay_delay(uint16 dly);



// USB_TO_SPI
#define USB_TO_SPI_BAUDRATE_MSK		0x1F
#define USB_TO_SPI_CPOL_MSK			0x20
#define USB_TO_SPI_CPHA_MSK			0x40
#define USB_TO_SPI_MSB_FIRST		0x80

RESULT usbtospi_init(void);
RESULT usbtospi_fini(void);
RESULT usbtospi_config(uint8 interface_index, uint16 freq, uint8 cpol, uint8 cpha, uint8 firstbit);
RESULT usbtospi_io(uint8 interface_index, uint8 *out, uint8 *in, uint16 outlen, uint16 inpos, uint16 inlen);



// USB_TO_GPIO
#define USB_TO_GPIO_DIR_MSK			0x01

RESULT usbtogpio_init(void);
RESULT usbtogpio_fini(void);
RESULT usbtogpio_config(uint8 interface_index, uint16 mask, uint16 direction);
RESULT usbtogpio_in(uint8 interface_index, uint16 mask, uint16 *value);
RESULT usbtogpio_out(uint8 interface_index, uint16 mask, uint16 value);



// USB_TO_ISSP
// read or write
#define ISSP_VECTOR_R			1
#define ISSP_VECTOR_W			0
// target
#define ISSP_VECTOR_SRAM		0
#define ISSP_VECTOR_CPUBANK		1
// Prog Mode
#define ISSP_PM_RESET			(1 << 0)
#define ISSP_PM_POWER_ON		(0 << 0)
// wap result
#define ISSP_WAP_OK				0x00
#define ISSP_WAP_TIMEOUT		0x01

RESULT usbtoissp_init(void);
RESULT usbtoissp_fini(void);
RESULT usbtoissp_config(uint8 interface_index);
RESULT usbtoissp_enter_program_mode(uint8 interface_index, uint8 mode);
RESULT usbtoissp_leave_program_mode(uint8 interface_index, uint8 mode);
RESULT usbtoissp_wait_and_poll(uint8 interface_index);
RESULT usbtoissp_vector(uint8 interface_index, uint8 operate, uint8 addr, uint8 data, uint8 *buf);



// USB_TO_JTAG_HL
RESULT usbtojtaghl_init(void);
RESULT usbtojtaghl_fini(void);
RESULT usbtojtaghl_config(uint8 interface_index, uint16 kHz, uint8 ub, uint8 ua, uint16 bb, uint16 ba);
RESULT usbtojtaghl_ir(uint8 interface_index, uint8 *ir, uint16 bitlen, uint8 idle, uint8 want_ret);
RESULT usbtojtaghl_dr(uint8 interface_index, uint8 *dr, uint16 bitlen, uint8 idle, uint8 want_ret);
RESULT usbtojtaghl_tmsbyte(uint8 interface_index, uint8 *tms, uint8 bytelen);

