/**************************************************************************
 *  Copyright (C) 2008 by Simon Qian                                      *
 *  SimonQian@SimonQian.com                                               *
 *                                                                        *
 *  Project:    vsprog                                                    *
 *  File:       versaloon.h                                               *
 *  Author:     SimonQian                                                 *
 *  Versaion:   See changelog                                             *
 *  Purpose:    common functions Versaloon                                *
 *  License:    See license                                               *
 *------------------------------------------------------------------------*
 *  Change Log:                                                           *
 *      YYYY-MM-DD:     What(by Who)                                      *
 *      2008-10-29:     created(by SimonQian)                             *
 **************************************************************************/
#include "usb.h"
#include "usbapi.h"

#ifndef __VERSALOON_H__
#define __VERSALOON_H__

#define VERSALOON_STRING				"versaloon"

#define VERSALOON_VID					0x03EB
#define VERSALOON_PID					0x2103
#define VERSALOON_OUTP					0x02
#define VERSALOON_INP					0x82

#define VERSALOON_FULL					1
#define VERSALOON_MINI					2
#define VERSALOON_NANO					3

#define VERSALOON_TIMEOUT				1000

// USB Commands
// Common Commands
#define VERSALOON_COMMON_CMD_START		0x00
#define VERSALOON_COMMON_CMD_END		0x0F

#define VERSALOON_GET_INFO				0x00
#define VERSALOON_GET_TVCC				0x01
#define VERSALOON_GET_HARDWARE			0x02
#define VERSALOON_GET_OFFLINE_SIZE		0x08
#define VERSALOON_ERASE_OFFLINE_DATA	0x09
#define VERSALOON_WRITE_OFFLINE_DATA	0x0A
#define VERSALOON_GET_OFFLINE_CHECKSUM	0x0B
#define VERSALOON_FW_UPDATE				0x0F

// MCU Command
#define VERSALOON_MCU_CMD_START			0x10
#define VERSALOON_MCU_CMD_END			0x1F

// USB_TO_XXX Command
#define VERSALOON_USB_TO_XXX_CMD_START	0x20
#define VERSALOON_USB_TO_XXX_CMD_END	0x7F

// OLink Command
#define VERSALOON_OLINK_CMD_START		0x80
#define VERSALOON_OLINK_CMD_END			0xFF



// Mass-product
#define MP_OK							0x00
#define MP_FAIL							0x01

#define MP_ISSP							0x11



// interfaces
#define VERSALOON_SPI_PORT				0
#define VERSALOON_GPIO_PORT				0
#define VERSALOON_ISSP_PORT				0
#define VERSALOON_JTAGHL_PORT			0



// pending struct
#define VERSALOON_MAX_PENDING_NUMBER	4096//1024
typedef struct
{
	uint8 type;
	uint8 cmd;
	uint16 want_data_pos;
	uint16 want_data_size;
	uint16 actual_data_size;
	uint8 *data_buffer;
	uint8 collect;
}versaloon_pending_result_t;
extern versaloon_pending_result_t versaloon_pending_result[VERSALOON_MAX_PENDING_NUMBER];
extern uint16 versaloon_pending_idx;
RESULT versaloon_add_pending(uint8 type, uint8 cmd, uint16 actual_szie, uint16 want_pos, uint16 want_size, uint8 *buffer, uint8 collect);




RESULT versaloon_check_argument(char cmd, const char **argu, uint32 argu_num);
RESULT versaloon_init_capability(void *p);

void versaloon_set_usb_param(uint16 vid, uint16 pid, uint8 epin, uint8 epout);
RESULT versaloon_get_target_voltage(uint16 *voltage);
RESULT versaloon_get_hardware(uint8 *hardware);
const char* versaloon_get_hardware_name(uint8 idx);
RESULT versaloon_send_command(uint16 out_len, uint16 *inlen);
RESULT versaloon_init(void);
RESULT versaloon_fini(void);

RESULT versaloon_download_mass_product_data(uint8 *buffer, uint32 len);
RESULT versaloon_query_mass_product_data_size(uint32 *size);

extern uint8 *versaloon_buf;
extern uint16 versaloon_buf_size;

#endif

