/**************************************************************************
 *  Copyright (C) 2008 by Simon Qian                                      *
 *  SimonQian@SimonQian.com                                               *
 *                                                                        *
 *  Project:    vsprog                                                    *
 *  File:       target.c                                                  *
 *  Author:     SimonQian                                                 *
 *  Versaion:   See changelog                                             *
 *  Purpose:    common functions for all targets                          *
 *  License:    See license                                               *
 *------------------------------------------------------------------------*
 *  Change Log:                                                           *
 *      YYYY-MM-DD:     What(by Who)                                      *
 *      2008-10-29:     created(by SimonQian)                             *
 **************************************************************************/
#include "app_cfg.h"

#include "at89s5x/at89s5x.h"

uint8 page_buf[TARGET_PAGE_BUF_SIZE];

target_info_t targets_info[] = 
{
	// at89s5x
	{
		S5X_STRING,								// name
		APPLICATION | LOCK,						// areas
		s5x_program_area_map,					// program_area_map
		SPI | GPIO,								// interfaces_needed
		s5x_parse_argument,						// parse_argument
		s5x_probe_chip,							// probe_chip
		s5x_init,								// init
		s5x_fini,								// fini
		s5x_prepare_buffer,						// prepare_buffer
		s5x_write_buffer_from_file_callback,	// write_buffer_from_file_callback
		NULL,//s5x_write_file_from_buffer_callback,	// write_file_from_buffer_callback
		s5x_program,							// program

//		NULL,									// get_mass_product_data_size
//		NULL,									// prepare_mass_product_data
	},
	{ NULL }
};
target_info_t *cur_target = NULL;

void target_print_list(void)
{
	uint32 i;

	printf("Supported targets:\n");
	for(i = 0; i < (sizeof(targets_info) / sizeof(targets_info[0]) - 1); i++)
	{
		targets_info[i].parse_argument('S', NULL, 0);
	}
	printf("\n");
}

uint32 target_get_number(void)
{
	return sizeof(targets_info) / sizeof(targets_info[0]) - 1;
}

RESULT target_init(program_info_t *pi)
{
	uint32 i;

	if((NULL == pi->chip_name) && (NULL == pi->chip_type))
	{
		return ERROR_FAIL;
	}

	if(NULL == pi->chip_type)
	{
		// find which series of target contain current chip_name
		for(i = 0; i < (sizeof(targets_info) / sizeof(targets_info[0]) - 1); i++)
		{
			if(ERROR_OK == targets_info[i].probe_chip(pi->chip_name))
			{
				cur_target = &targets_info[i];
				pi->chip_type = (char *)targets_info[i].name;
				LOG_INFO("%s initialized for %s\n", cur_target->name, pi->chip_name);

				return ERROR_OK;
			}
		}
		LOG_ERROR("%s not supported\n", pi->chip_name);
	}
	else
	{
		// find current series of chip_type
		for(i = 0; i < (sizeof(targets_info) / sizeof(targets_info[0]) - 1); i++)
		{
			if(!strcmp(targets_info[i].name, pi->chip_type))
			{
				if((pi->chip_name != NULL) && (ERROR_OK != targets_info[i].probe_chip(pi->chip_name)))
				{
					LOG_ERROR("%s not supported by %s\n", pi->chip_name, targets_info[i].name);

					return ERROR_FAIL;
				}
				else
				{
					cur_target = &targets_info[i];
					LOG_INFO("%s initialized\n", cur_target->name);

					return ERROR_OK;
				}
			}
		}
		LOG_ERROR("%s not supported\n", pi->chip_type);
	}

	cur_target = NULL;

	return ERROR_FAIL;
}

