/**************************************************************************
 *  Copyright (C) 2008 by Simon Qian                                      *
 *  SimonQian@SimonQian.com                                               *
 *                                                                        *
 *  Project:    vsprog                                                    *
 *  File:       target.h                                                  *
 *  Author:     SimonQian                                                 *
 *  Versaion:   See changelog                                             *
 *  Purpose:    common functions for all targets                          *
 *  License:    See license                                               *
 *------------------------------------------------------------------------*
 *  Change Log:                                                           *
 *      YYYY-MM-DD:     What(by Who)                                      *
 *      2008-10-29:     created(by SimonQian)                             *
 **************************************************************************/
typedef struct
{
	const char *name;
	const uint32 areas;
	const program_area_map_t *program_area_map;
	const uint32 interfaces_needed;

	RESULT (*parse_argument)(char cmd, const char **argu, uint32 argu_num);

	RESULT (*probe_chip)(char *chip_name);
	RESULT (*init)(program_info_t *pi, const char *dir, programmer_info_t *prog);
	RESULT (*fini)(void);
	RESULT (*prepare_buffer)(program_info_t pi);
	RESULT (*write_buffer_from_file_callback)(uint32 address, uint32 seg_addr, uint8* data, uint32 length);
	RESULT (*write_file_from_buffer_callback)();
	RESULT (*program)(operation_t operations, program_info_t pi, programmer_info_t *prog);

	RESULT (*get_mass_product_data_size)(operation_t operations, program_info_t pi, uint32 *size);
	RESULT (*prepare_mass_product_data)(operation_t operations, program_info_t pi, uint8 *buff);
}target_info_t;

// gpio pins
#define GPIO_SRST				(1 << 0)

// jtag pins
#define JTAG_SRST				(1 << 0)
#define JTAG_TRST				(1 << 1)
#define JTAG_USR1				(1 << 2)
#define JTAG_USR2				(1 << 3)

// 


#define TARGET_PAGE_BUF_SIZE	4096
extern target_info_t *cur_target;
extern target_info_t targets_info[];
extern uint8 page_buf[TARGET_PAGE_BUF_SIZE];

void target_print_list(void);
uint32 target_get_number(void);
RESULT target_init(program_info_t *pi);

