/**************************************************************************
 *  Copyright (C) 2008 by Simon Qian                                      *
 *  SimonQian@SimonQian.com                                               *
 *                                                                        *
 *  Project:    vsprog                                                    *
 *  File:       pgbar.c                                                   *
 *  Author:     SimonQian                                                 *
 *  Versaion:   See changelog                                             *
 *  Purpose:    progressbar for CLI                                       *
 *  License:    See license                                               *
 *------------------------------------------------------------------------*
 *  Change Log:                                                           *
 *      YYYY-MM-DD:     What(by Who)                                      *
 *      2008-10-29:     created(by SimonQian)                             *
 **************************************************************************/
#include "app_cfg.h"

char *end_str = NULL, disp_char = 0;
int32 min_num = 0, max_num = 0, position = 0;
uint32 max_num_of_chars = 0;
#if _PLATFORM == MSWIN
clock_t start_time, end_time;
#elif _PLATFORM == LINUX
struct timeval start_time, end_time;
#endif

uint32 pgbar_get_num_of_chars(int32 pos)
{
	return (pos - min_num) * max_num_of_chars / (max_num - min_num);
}

void pgbar_update(uint32 step)
{
	uint32 noc;

	// erase previous chars
	noc = 3;		// 3 is "%xx"
	while(noc-- > 0)
	{
		if(0 == gui_mode)
		{
			printf("\b");
		}
	}
	noc = pgbar_get_num_of_chars(position);

	position += step;
	if(position > max_num)
	{
		position = max_num;
	}
	else if(position < min_num)
	{
		position = min_num;
	}

	// output new chars
	noc = pgbar_get_num_of_chars(position) - noc;
	while(noc-- > 0)
	{
		printf("%c", disp_char);
	}

	// output percentage
	printf("%%%02d", pgbar_get_num_of_chars(position) * 100 / max_num_of_chars);

	if(1 == gui_mode)
	{
		printf("\n");
	}
	fflush(stdout);
}

void pgbar_init(char *s, char *e, uint32 min, uint32 max, uint32 max_chars, char c)
{
	end_str = e;
	max_num = max;
	position = min_num = min;
	max_num_of_chars = max_chars;
	disp_char = c;

	if(NULL != s)
	{
		printf("%s", s);
	}
	printf("%%00");

	pgbar_update(0);

	// init timer
#if _PLATFORM == MSWIN
	start_time = clock();
#elif _PLATFORM == LINUX
	gettimeofday(&start_time, NULL);
#endif

	fflush(stdout);
}

void pgbar_fini(void)
{
	if(NULL != end_str)
	{
		printf("%s ", end_str);
	}

	// finished, print time used
#if _PLATFORM == MSWIN
	end_time = clock();
	printf("%02.02fs used\n", (float)(end_time - start_time) / CLOCKS_PER_SEC);
#elif _PLATFORM == LINUX
	gettimeofday(&end_time, NULL);
	printf("%02.02fs used\n", (float)(end_time.tv_sec - start_time.tv_sec) + (float)(end_time.tv_usec - start_time.tv_usec) / 1000000);
#endif

	fflush(stdout);
}

