/**************************************************************************
 *  Copyright (C) 2008 by Simon Qian                                      *
 *  SimonQian@SimonQian.com                                               *
 *                                                                        *
 *  Project:    vsprog                                                    *
 *  File:       vsprog.c                                                  *
 *  Author:     SimonQian                                                 *
 *  Versaion:   See changelog                                             *
 *  Purpose:    main .c file                                              *
 *  License:    See license                                               *
 *------------------------------------------------------------------------*
 *  Change Log:                                                           *
 *      YYYY-MM-DD:     What(by Who)                                      *
 *      2008-10-29:     created(by SimonQian)                             *
 **************************************************************************/
#include "app_cfg.h"

int verbosity = LOG_DEFAULT_LEVEL;

program_info_t program_info;
operation_t operations;
uint8 gui_mode = 0;

FILE *hex_file = NULL;

static void free_all_and_exit(int exit_code)
{
	if(cur_programmer != NULL)
	{
		cur_programmer->fini();
	}

	// free buffer
	if(program_info.boot != NULL)
	{
		free(program_info.boot);
		program_info.boot = NULL;
	}
	if(program_info.boot_checksum != NULL)
	{
		free(program_info.boot_checksum);
		program_info.boot_checksum = NULL;
	}
	if(program_info.app != NULL)
	{
		free(program_info.app);
		program_info.app = NULL;
	}
	if(program_info.app_checksum != NULL)
	{
		free(program_info.app_checksum);
		program_info.app_checksum = NULL;
	}
	if(program_info.otp_rom != NULL)
	{
		free(program_info.otp_rom);
		program_info.otp_rom = NULL;
	}
	if(program_info.otp_rom_checksum != NULL)
	{
		free(program_info.otp_rom_checksum);
		program_info.otp_rom_checksum = NULL;
	}
	if(program_info.eeprom != NULL)
	{
		free(program_info.eeprom);
		program_info.eeprom = NULL;
	}
	if(program_info.eeprom_checksum != NULL)
	{
		free(program_info.eeprom_checksum);
		program_info.eeprom_checksum = NULL;
	}
	if(program_info.fuse != NULL)
	{
		free(program_info.fuse);
		program_info.fuse = NULL;
	}
	if(program_info.fuse_checksum != NULL)
	{
		free(program_info.fuse_checksum);
		program_info.fuse_checksum = NULL;
	}
	if(program_info.lock != NULL)
	{
		free(program_info.lock);
		program_info.lock = NULL;
	}
	if(program_info.lock_checksum != NULL)
	{
		free(program_info.lock_checksum);
		program_info.lock_checksum = NULL;
	}
	if(program_info.user_area != NULL)
	{
		free(program_info.user_area);
		program_info.user_area = NULL;
	}
	if(program_info.user_area_checksum != NULL)
	{
		free(program_info.user_area_checksum);
		program_info.user_area_checksum = NULL;
	}

	exit(exit_code);
}

static RESULT malloc_program_buffer(void)
{
	if((NULL == program_info.boot) && (program_info.boot_size > 0))
	{
		program_info.boot = malloc(program_info.boot_size);
		if(NULL == program_info.boot) { return ERROR_FAIL; }
	}
	if((NULL == program_info.boot_checksum) && (program_info.boot_checksum_size > 0))
	{
		program_info.boot_checksum = malloc(program_info.boot_checksum_size);
		if(NULL == program_info.boot_checksum) { return ERROR_FAIL; }
	}
	if((NULL == program_info.app) && (program_info.app_size > 0))
	{
		program_info.app = malloc(program_info.app_size);
		if(NULL == program_info.app) { return ERROR_FAIL; }
	}
	if((NULL == program_info.app_checksum) && (program_info.app_checksum_size > 0))
	{
		program_info.app_checksum = malloc(program_info.app_checksum_size);
		if(NULL == program_info.app_checksum) { return ERROR_FAIL; }
	}
	if((NULL == program_info.eeprom) && (program_info.eeprom_size > 0))
	{
		program_info.eeprom = malloc(program_info.eeprom_size);
		if(NULL == program_info.eeprom) { return ERROR_FAIL; }
	}
	if((NULL == program_info.eeprom_checksum) && (program_info.eeprom_checksum_size > 0))
	{
		program_info.eeprom_checksum = malloc(program_info.eeprom_checksum_size);
		if(NULL == program_info.eeprom_checksum) { return ERROR_FAIL; }
	}
	if((NULL == program_info.otp_rom) && (program_info.otp_rom_size > 0))
	{
		program_info.otp_rom = malloc(program_info.otp_rom_size);
		if(NULL == program_info.otp_rom) { return ERROR_FAIL; }
	}
	if((NULL == program_info.otp_rom_checksum) && (program_info.otp_rom_checksum_size > 0))
	{
		program_info.otp_rom_checksum = malloc(program_info.otp_rom_checksum_size);
		if(NULL == program_info.otp_rom_checksum) { return ERROR_FAIL; }
	}
	if((NULL == program_info.fuse) && (program_info.fuse_size > 0))
	{
		program_info.fuse = malloc(program_info.fuse_size);
		if(NULL == program_info.fuse) { return ERROR_FAIL; }
	}
	if((NULL == program_info.fuse_checksum) && (program_info.fuse_checksum_size > 0))
	{
		program_info.fuse_checksum = malloc(program_info.fuse_checksum_size);
		if(NULL == program_info.fuse_checksum) { return ERROR_FAIL; }
	}
	if((NULL == program_info.lock) && (program_info.lock_size > 0))
	{
		program_info.lock = malloc(program_info.lock_size);
		if(NULL == program_info.lock) { return ERROR_FAIL; }
	}
	if((NULL == program_info.lock_checksum) && (program_info.lock_checksum_size > 0))
	{
		program_info.lock_checksum = malloc(program_info.lock_checksum_size);
		if(NULL == program_info.lock_checksum) { return ERROR_FAIL; }
	}
	if((NULL == program_info.user_area) && (program_info.user_area_size > 0))
	{
		program_info.user_area = malloc(program_info.user_area_size);
		if(NULL == program_info.user_area) { return ERROR_FAIL; }
	}
	if((NULL == program_info.user_area_checksum) && (program_info.user_area_checksum_size > 0))
	{
		program_info.user_area_checksum = malloc(program_info.user_area_checksum_size);
		if(NULL == program_info.user_area_checksum) { return ERROR_FAIL; }
	}

	return ERROR_OK;
}

static void usage_and_exit(int exit_code)
{
	printf("usage: vsprog [-sSERIES] [-cCHIPNAME] [-pPROGRAMMER] (-o(e|r|w|v|c)[b][a][e][o][f][l][u]) [-v[n]] [-G] [-h[OBJECT]] [-a] [-S[OBJECT]] [-i HEX_FILE]\n"
			"  <-sSERIES> Series of the target\n"
			"  <-pPROGRAMMER> set Programmer\n"
			"  <-cCHIPNAME> Chip name, auto-detect if not defined\n"
			"  <-M> set Mass-product\n"
			"  <-o(e|r|w|v|c)[b][a][e][o][f][l][u]> Operations\n"
			"    (e|r|w|v|c) (Erase|Read|Write|Verify|Checksum)\n"
			"    [b][a][e][o][f][l][u] [Bootloader][Application][Eeprom][Otp_rom][Fuse][Lock][User_area]\n"
			"  <-v[n]> Verbosity level(0..3)\n"
			"  <-G> Gui mode\n"
			"  <-h[OBJECT]> show Help of OBJECT\n"
			"  <-a> show About\n"
			"  <-S[OBJECT]> show Supported device of OBJRECT\n"
			"  <-i HEX_FILE> Input file\n"
		);
	free_all_and_exit(exit_code);
}

static void about_and_exit(int exit_code)
{
	printf("about:\n"
			"vsprog is VerSaloon Programmer.\n"
			"author:\n"
			"SimonQian(SimonQian@SimonQian.com)\n\n"
			"contributor:\n");
	free_all_and_exit(exit_code);
}

static uint32 parse_option_command_and_args(int argc, char **argv, int* num_of_args, char* command, char*** option_args)
{
	static char* buf[256];
	int i = 0, j, option_length;

#ifdef PARAM_CHECK
	if((NULL == command) || (NULL == num_of_args) || (NULL == option_args) || (NULL == argv)) { return 0; }
#endif

	option_length = 0;
	*command = 0;
	*num_of_args = 0;

	if((strlen(argv[0]) >= 2) && ('-' == argv[0][0]) && (argv[0][1] != 0))
	{
		option_length = 1;
		j = 1;

		*command = argv[0][1];
		if(argv[0][2] != 0)
		{
			buf[i++] = argv[0] + 2;
		}

		while(((j + 1) <= argc) && (argv[j][0] != '-'))
		{
			if(i < sizeof(buf))
			{
				buf[i++] = argv[j++];
			}
			option_length++;
		}
	}
	*option_args = buf;
	*num_of_args = i;

	return option_length;
}

static RESULT parse_operation(uint32 *operation, uint32 *data_pos, const char *opt, uint32 optlen)
{
	uint32 i, j, mask = 0;

#ifdef PARAM_CHECK
	if((NULL == operation) || (NULL == opt)) { LOG_BUG("parameter invalid\n"); return ERROR_FAIL; }
#endif

	if(data_pos != NULL)
	{
		*data_pos = 0;
	}

	for(i = 0; i < optlen; i++)
	{
		if(cur_target != NULL)
		{
			j = 0;
			while(cur_target->program_area_map[j].area_mask > 0)
			{
				if(cur_target->program_area_map[j].area_char == opt[i])
				{
					mask |= cur_target->program_area_map[j].area_mask;
					if((data_pos != NULL) && (cur_target->program_area_map[j].data_pos))
					{
						(*data_pos)++;
					}
					break;
				}
				j++;
			}
			if(cur_target->program_area_map[j].area_mask > 0)
			{
				continue;
			}
			else
			{
				LOG_ERROR("%c is not valid for target\n", opt[i]);
				return ERROR_FAIL;
			}
		}
		else
		{
			switch(opt[i])
			{
			case 'b':
				mask |= BOOTLOADER;
				break;
			case 'a':
				mask |= APPLICATION;
				break;
			case 'e':
				mask |= EEPROM;
				break;
			case 'o':
				mask |= OTP_ROM;
				break;
			case 'f':
				mask |= FUSE;
				break;
			case 'l':
				mask |= LOCK;
				break;
			case 'u':
				mask |= USER_SIG;
				break;
			default:
				LOG_ERROR("%c is not valid for target", opt[i]);
				return ERROR_FAIL;
				break;
			}
		}
	}

	*operation = mask;

	return ERROR_OK;
}

int main(int argc, char* argv[])
{
	int i, tmp, num_of_args = 0;
	char command = 0, **option_args = NULL, *p;
	char app_dir[MAX_PATH], *filename = NULL;

	uint32 data_pos_read = 0, data_pos_write = 0, data_pos_tmp, j;
	uint8 mp = 0, *mp_buf = NULL;
	uint32 mp_size = 0, tmp_size = 0;

	printf("vsprog(2008-09-18) by SimonQian, compiled on " __DATE__ "\n"
			"URL: http://www.SimonQian.com/en/Versaloon\n"
			"mail:simonqian@simonqian.com\n\n");

	// get directory of the application
	strcpy(app_dir, argv[0]);
	p = strrchr(app_dir, FILE_SEPARATOR);
	if(NULL == p)
	{
		app_dir[0] = 0;
	}
	else
	{
		*(p + 1) = 0;
		p = NULL;
	}

	// clear memory
	memset(&program_info, 0, sizeof(program_info));
	// select default programmer
	programmer_init(NULL, app_dir);

	if(1 == argc) { usage_and_exit(0); }

	// parse args
	i = 1;
	do{
		tmp = parse_option_command_and_args(argc - i, argv + i, &num_of_args, &command, &option_args);
		if(0 == tmp) { LOG_ERROR("args error\n"); usage_and_exit(1); }

		i += tmp;

		switch(command)
		{
		case 0:
			break;
		case 'h':
			// Help
			if(0 == num_of_args)
			{
				usage_and_exit(0);
			}
			else if(1 == num_of_args)
			{
				tmp = 0;
				while(programmers_info[tmp].name != NULL)
				{
					if(!strcmp(programmers_info[tmp].name, strlwr(option_args[0])))
					{
						programmers_info[tmp].parse_argument('h', NULL, 0);
						exit(0);
					}
					tmp++;
				}

				tmp = 0;
				while(targets_info[tmp].name != NULL)
				{
					if(!strcmp(targets_info[tmp].name, strlwr(option_args[0])))
					{
						targets_info[tmp].parse_argument('h', NULL, 0);
						exit(0);
					}
					tmp++;
				}

				LOG_ERROR("%s not supported\n", option_args[0]);
				exit(1);
			}
			else
			{
				LOG_ERROR("args of %c error\n", command);
				usage_and_exit(1);
			}
			break;
		case 'a':
			// About
			about_and_exit(0);
			break;
		case 'i':
			if(num_of_args != 1) { LOG_ERROR("args of %c error\n", command); usage_and_exit(1); }
			if((('"' == option_args[0][0]) && ('"' == option_args[0][strlen(option_args[0]) - 1])) || 
			   (('\'' == option_args[0][0]) && ('\'' == option_args[0][strlen(option_args[0]) - 1])))
			{
				option_args[0][strlen(option_args[0]) - 1] = '\0';
				strcpy(option_args[0], option_args[0] + 1);
			}
			num_of_args = (int)strlen(option_args[0]);
			filename = (char *)malloc(num_of_args + 1);
			if(NULL == filename) { LOG_ERROR("not enough memory.\n"); free_all_and_exit(1); }
			strncpy(filename, option_args[0], num_of_args + 1);
			break;
		case 'S':
			if(0 == num_of_args)
			{
				// print all Supported programmers
				programmer_print_list();
				// print all Supported devices
				target_print_list();
				exit(0);
			}
			else if(1 == num_of_args)
			{
				tmp = 0;
				while(programmers_info[tmp].name != NULL)
				{
					if(!strcmp(programmers_info[tmp].name, strlwr(option_args[0])))
					{
						programmers_info[tmp].parse_argument('S', NULL, 0);
						exit(0);
					}
					tmp++;
				}

				tmp = 0;
				while(targets_info[tmp].name != NULL)
				{
					if(!strcmp(targets_info[tmp].name, strlwr(option_args[0])))
					{
						targets_info[tmp].parse_argument('S', NULL, 0);
						exit(0);
					}
					tmp++;
				}

				LOG_ERROR("%s not supported\n", option_args[0]);
				exit(1);
			}
			else
			{
				LOG_ERROR("args of %c error\n", command);
				usage_and_exit(1);
			}
			break;
		case 'c':
			// Chip name
			if(num_of_args != 1) { LOG_ERROR("args of %c error\n", command); usage_and_exit(1); }
			strlwr(option_args[0]);
			program_info.chip_name = option_args[0];
			if(ERROR_OK != target_init(&program_info)) { LOG_ERROR("target initialize failed\n"); usage_and_exit(1); }
			break;
		case 's':
			// chip Series
			if(num_of_args != 1) { LOG_ERROR("args of %c error\n", command); usage_and_exit(1); }
			strlwr(option_args[0]);
			program_info.chip_type = option_args[0];
			if(ERROR_OK != target_init(&program_info)) { LOG_ERROR("target initialize failed\n"); usage_and_exit(1); }
			break;
		case 'M':
			// Mass-product
			mp = 1;
			break;
		case 'p':
			// Programmer
			if(num_of_args != 1) { LOG_ERROR("args of %c error\n", command); usage_and_exit(1); }
			strlwr(option_args[0]);
			if(ERROR_OK != programmer_init(option_args[0], app_dir)) { LOG_ERROR("%s not supported\n", option_args[0]); usage_and_exit(1); }
			break;
		case 'v':
			// Verbosity
			if((num_of_args > 1) || 
				((1 == num_of_args) && 
					((strlen(option_args[0]) != 1) || 
					(option_args[0][0] < '0') || 
					(option_args[0][0] > '9')))) { LOG_ERROR("args of %c error\n", command); usage_and_exit(1); }
			if(0 == num_of_args)
			{
				verbosity = 1;
			}
			else
			{
				verbosity = option_args[0][0] - '0';
			}
			break;
		case 'G':
			if(num_of_args != 0) { LOG_ERROR("args of %c error\n", command); usage_and_exit(1); }
			gui_mode = 1;
			break;
		case 'o':
			// Operations
			if(num_of_args != 1) { LOG_ERROR("args of %c error\n", command); usage_and_exit(1); }
			tmp = (int)strlen(option_args[0]);
			switch(option_args[0][0])
			{
			case 'e':
				// Erase
				if(tmp > NUM_OF_TARGET_AREA) { LOG_ERROR("args of %c error\n", command); usage_and_exit(1); }
				if(operations.erase_operations != 0) { LOG_ERROR("erase operation has been define\n"); usage_and_exit(1); }
				if(1 == tmp)
				{
					operations.erase_operations |= ALL;
				}
				else
				{
					if(ERROR_OK != parse_operation(&operations.erase_operations, NULL, option_args[0] + 1, tmp - 1)) { LOG_ERROR("erase option error\n"); usage_and_exit(1); }
				}
				break;
			case 'r':
				// Read
				if(tmp > NUM_OF_TARGET_AREA) { LOG_ERROR("args of %c error\n", command); usage_and_exit(1); }
				if(operations.read_operations != 0) { LOG_ERROR("read operation has been define\n"); usage_and_exit(1); }
				if(1 == tmp)
				{
					for(j = 0; cur_target->program_area_map[j].area_mask > 0; j++)
					{
						operations.read_operations |= cur_target->program_area_map[j].area_mask;
						data_pos_read += cur_target->program_area_map[j].data_pos;
					}
				}
				else
				{
					if(ERROR_OK != parse_operation(&operations.read_operations, &data_pos_tmp, option_args[0] + 1, tmp - 1)) { LOG_ERROR("erase option error\n"); usage_and_exit(1); }
					data_pos_read += data_pos_tmp;
				}
				break;
			case 'v':
				// Verify
				if(tmp > NUM_OF_TARGET_AREA) { LOG_ERROR("args of %c error\n", command); usage_and_exit(1); }
				if(operations.verify_operations != 0) { LOG_ERROR("verify operation has been define\n"); usage_and_exit(1); }
				if(1 == tmp)
				{
					for(j = 0; cur_target->program_area_map[j].area_mask > 0; j++)
					{
						operations.read_operations |= cur_target->program_area_map[j].area_mask;
						data_pos_write += cur_target->program_area_map[j].data_pos;
					}
				}
				else
				{
					if(ERROR_OK != parse_operation(&operations.read_operations, &data_pos_tmp, option_args[0] + 1, tmp - 1)) { LOG_ERROR("erase option error\n"); usage_and_exit(1); }
					data_pos_write += data_pos_tmp;
				}
				operations.verify_operations = operations.read_operations;
				break;
			case 'w':
				// Write
				if(tmp > NUM_OF_TARGET_AREA) { LOG_ERROR("args of %c error\n", command); usage_and_exit(1); }
				if(operations.write_operations != 0) { LOG_ERROR("write operation has been define\n"); usage_and_exit(1); }
				if(1 == tmp)
				{
					for(j = 0; cur_target->program_area_map[j].area_mask > 0; j++)
					{
						operations.write_operations |= cur_target->program_area_map[j].area_mask;
						data_pos_write += cur_target->program_area_map[j].data_pos;
					}
				}
				else
				{
					if(ERROR_OK != parse_operation(&operations.write_operations, &data_pos_tmp, option_args[0] + 1, tmp - 1)) { LOG_ERROR("erase option error\n"); usage_and_exit(1); }
					data_pos_write += data_pos_tmp;
				}
				break;
			default:
				LOG_ERROR("args of %c error\n", command);
				usage_and_exit(1);
				break;
			}
			break;
		default:
			if(cur_target != NULL)
			{
				if(ERROR_OK == cur_target->parse_argument(command, (const char **)option_args, num_of_args))
				{
					break;
				}
			}
			if(cur_programmer != NULL)
			{
				if(ERROR_OK == cur_programmer->parse_argument(command, (const char **)option_args, num_of_args))
				{
					break;
				}
			}

			LOG_ERROR("-%c option error\n", command);
			usage_and_exit(1);
			break;
		}
	}while(i < argc);

	// init programmer capabilities
	cur_programmer->init_capability(cur_programmer);

	if(NULL == cur_target) { LOG_ERROR("target chip not defined, use -c and -s\n"); free_all_and_exit(1); }
	// init and check programmer's ability
	if((cur_programmer->interfaces & cur_target->interfaces_needed) != cur_target->interfaces_needed)
	{
		LOG_ERROR("%s can not support %s\n", cur_programmer->name, cur_target->name);
		free_all_and_exit(1);
	}

	// open file
	if((data_pos_read > 0) || (data_pos_write > 0))
	{
		if((filename != NULL) && (strlen(filename) > 0))
		{
			LOG_DEBUG("open file: %s\n", filename);

			if((operations.write_operations > 0) || (operations.verify_operations > 0))
			{
				// open file for read
				hex_file = fopen(filename, "rt");
			}
			else if((operations.read_operations > 0) && (0 == operations.verify_operations))
			{
				// open file for write
				hex_file = fopen(filename, "wt");
			}
			if(NULL == hex_file)
			{
				LOG_ERROR("could not open %s\n", filename);
				free(filename);
				filename = NULL;
				free_all_and_exit(1);
			}
			free(filename);
			filename = NULL;
		}
		else
		{
			LOG_ERROR("input hex file not defined, use -i\n");
			free_all_and_exit(1);
		}
	}

	// init programmer
	if(ERROR_OK != cur_programmer->init()) { LOG_ERROR("programmer %s not initialized\n", cur_programmer->name); free_all_and_exit(1); }

	// init target
	if(ERROR_OK != cur_target->init(&program_info, app_dir, cur_programmer)) { LOG_ERROR("target init failed\n"); free_all_and_exit(1); }

	// malloc buffer
	malloc_program_buffer();
	cur_target->prepare_buffer(program_info);
	if((data_pos_write > 0) && (hex_file != NULL))
	{
		if(ERROR_OK != read_hex_file(hex_file, cur_target->write_buffer_from_file_callback)) { LOG_ERROR("read file error\n"); free_all_and_exit(1); }
		fclose(hex_file);
		hex_file = NULL;
	}

	// do programming
	if(mp)
	{
		// mass-product
		if((operations.read_operations > 0) && (operations.read_operations != operations.verify_operations))
		{
			LOG_ERROR("read operation not supported in mass-product mode\n");
			free_all_and_exit(1);
		}
		if((NULL == cur_target->get_mass_product_data_size) || (NULL == cur_target->prepare_mass_product_data))
		{
			LOG_ERROR("mass-product of %s is not supported\n", cur_target->name);
			free_all_and_exit(1);
		}
		if((NULL == cur_programmer->query_mass_product_data_size) || (NULL == cur_programmer->download_mass_product_data))
		{
			LOG_ERROR("%s does not supported mass-product\n", cur_programmer->name);
			free_all_and_exit(1);
		}
		if(ERROR_OK != cur_target->get_mass_product_data_size(operations, program_info, &mp_size)) { LOG_ERROR("fail to get size of mass-product data\n"); free_all_and_exit(1); }
		if(ERROR_OK != cur_programmer->query_mass_product_data_size(&tmp_size)) { LOG_ERROR("fail to get size of mass-product area of %s\n", cur_programmer->name); free_all_and_exit(1); }

		mp_size += 1;		// 1 more byte for target index
		if(tmp_size >= mp_size)
		{
			mp_buf = (uint8*)malloc(mp_size);
			if(NULL == mp_buf) { LOG_ERROR("not enough memory\n"); free_all_and_exit(1); }

			if(ERROR_OK != cur_target->prepare_mass_product_data(operations, program_info, mp_buf + 1)) { LOG_ERROR("fail to prepare mass-product data\n"); free_all_and_exit(1); }
			// target index
			mp_buf[0] = (uint8)abs((int)((uint8*)cur_target - (uint8*)&targets_info)) / sizeof(targets_info[0]);
			if(ERROR_OK != cur_programmer->download_mass_product_data(mp_buf, mp_size)) { LOG_ERROR("fail to download mass-product data\n"); free_all_and_exit(1); }
		}
		else
		{
			LOG_ERROR("not enough mass-product area in %s\n", cur_programmer->name);
			free_all_and_exit(1);
		}
	}
	else
	{
		// isp
		if(operations.checksum_operations || operations.erase_operations || operations.read_operations || operations.verify_operations || operations.write_operations)
		{
			if(ERROR_OK != cur_target->program(operations, program_info, cur_programmer)) { LOG_ERROR("fail to operate %s\n", cur_target->name); free_all_and_exit(1); }
		}
	}

	// close handle and free memory
	free_all_and_exit(0);

	return 0;
}

