/**************************************************************************
 *  Copyright (C) 2008 by Simon Qian                                      *
 *  SimonQian@SimonQian.com                                               *
 *                                                                        *
 *  Project:    vsprog                                                    *
 *  File:       vsprog.c                                                  *
 *  Author:     SimonQian                                                 *
 *  Versaion:   See changelog                                             *
 *  Purpose:    main .h file                                              *
 *  License:    See license                                               *
 *------------------------------------------------------------------------*
 *  Change Log:                                                           *
 *      YYYY-MM-DD:     What(by Who)                                      *
 *      2008-10-29:     created(by SimonQian)                             *
 **************************************************************************/
#define CHIP_ID						(1 << 0)
#define BOOTLOADER					(1 << 1)
#define APPLICATION					(1 << 2)
#define EEPROM						(1 << 3)
#define OTP_ROM						(1 << 4)
#define FUSE						(1 << 5)
#define LOCK						(1 << 6)
#define USER_SIG					(1 << 7)
#define CHECKSUM					(1 << 8)
#define ALL							(1 << 31)
#define TARGET_AREA_MASK			(BOOTLOADER | APPLICATION | EEPROM | OTP_ROM | FUSE | LOCK | USER_SIG | CHECKSUM | ALL)
#define NUM_OF_TARGET_AREA			8

typedef struct
{
	char *chip_type;
	char *chip_name;
	uint32 target_index;

	uint8 *boot;
	uint32 boot_size;
	uint8 *boot_checksum;
	uint32 boot_checksum_size;

	uint8 *app;
	uint32 app_size;
	uint8 *app_checksum;
	uint32 app_checksum_size;

	uint8 *eeprom;
	uint32 eeprom_size;
	uint8 *eeprom_checksum;
	uint32 eeprom_checksum_size;

	uint8 *otp_rom;
	uint32 otp_rom_size;
	uint8 *otp_rom_checksum;
	uint32 otp_rom_checksum_size;

	uint8 *fuse;
	uint32 fuse_size;
	uint8 *fuse_checksum;
	uint32 fuse_checksum_size;

	uint8 *lock;
	uint32 lock_size;
	uint8 *lock_checksum;
	uint32 lock_checksum_size;

	uint8 *user_area;
	uint32 user_area_size;
	uint8 *user_area_checksum;
	uint32 user_area_checksum_size;

	uint8 *mass_product_data;
	uint32 mass_product_data_size;
}program_info_t;

typedef struct
{
	const char *chip_name;
	uint32 chip_id;
	uint32 program_mode;
	uint32 boot_page_size;
	uint32 boot_page_num;
	uint32 app_page_size;
	uint32 app_page_num;
	uint32 ee_page_size;
	uint32 ee_page_num;
	uint32 optrom_page_size;
	uint32 optrom_page_num;
	uint32 usrsig_page_size;
	uint32 usrsig_page_num;
	uint32 fuse_size;
	uint32 lock_size;
	uint32 boot_size;
	uint32 app_size;
	uint32 ee_size;
	uint32 optrom_size;
	uint32 usrsig_size;
}chip_param_t;

typedef struct
{
	uint32 erase_operations;
	uint32 write_operations;
	uint32 read_operations;
	uint32 verify_operations;
	uint32 checksum_operations;
}operation_t;

typedef struct
{
	uint32 area_mask;
	char area_char;
	uint8 data_pos;
}program_area_map_t;


// interfaces
#define USART						(1 << 0)
#define SPI							(1 << 1)
#define I2C							(1 << 2)
#define GPIO						(1 << 3)
#define CAN							(1 << 4)
#define CLOCK						(1 << 5)
#define ADC							(1 << 6)
#define DAC							(1 << 7)
#define POWER						(1 << 8)

#define ISSP						(1 << 16)
#define JTAG						(1 << 17)
#define SBW							(1 << 18)
#define C2							(1 << 19)
#define INTERFACES_MASK				(USART | SPI | I2C | GPIO | CAN | CLOCK | ADC | DAC | POWER | ISSP | JTAG)



// SPI
#define SPI_CPOL_HIGH				0x20
#define SPI_CPOL_LOW				0x00
#define SPI_CPHA_2EDGE				0x40
#define SPI_CPHA_1EDGE				0x00
#define SPI_MSB_FIRST				0x80
#define SPI_LSB_FIRST				0x00



// ISSP
#define ISSP_WAP_OK					0x00
#define ISSP_WAP_TIMEOUT			0x01

#define ISSP_VECTOR_ATTR_BANK		(1 << 0)
#define ISSP_VECTOR_ATTR_READ		(1 << 1)
#define ISSP_VECTOR_ATTR_APPENDBIT	(1 << 2)
#define ISSP_VECTOR_ATTR_0s			(1 << 3)



extern uint8 gui_mode;
extern int verbosity;

